/*
 * Copyright 2008,2010 Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

public class MenuForm
    extends Form
    implements CommandListener, ItemStateListener
{
    private final Command playCommand = new Command("Play new", Command.OK, 0);
    private final Command exitCommand = new Command("Exit", Command.EXIT, 0);
    private final Command continueCommand = new Command("Continue", Command.BACK, 1);

    private class OptDifficulty extends ChoiceGroup {
	OptDifficulty() {
	    super ("How Many Mines", Choice.EXCLUSIVE);
	    append("Very Few", null);
	    append("Few", null);
	    setSelectedIndex(append("Some", null), true);
	    append("Many", null);
	    append("A Lot", null);
	}

	public int howManyMines(ParticipantSelector parSel) {
	    return parSel.howManyMines(getSelectedIndex(), size());
	}
    }

    private abstract class SelCtorBuilder {
	public abstract String name();
	public abstract ParticipantSelector build();
	public boolean isDefault() { return false; }
    }

    private abstract class OptSelector extends ChoiceGroup {
	private final SelCtorBuilder[] _ctors;
	public OptSelector(SelCtorBuilder[] ctors) {
	    super("Neighborhood", Choice.EXCLUSIVE);
	    _ctors = ctors;
	    for (int i = 0; i < ctors.length; ++i) {
		int idx = append(ctors[i].name(), null);
		if (ctors[i].isDefault())
		    setSelectedIndex(idx, true);
	    }
	}

	public ParticipantSelector parSel() {
	    return _ctors[getSelectedIndex()].build();
	}
    }

    private class OptSelector8 extends OptSelector {
	OptSelector8() {
	    super(new SelCtorBuilder[] {
		    new SelCtorBuilder() {
			public String name() { return "4"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel4();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "8"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel8();
			}
			public boolean isDefault() { return true; }
		    },
		    new SelCtorBuilder() {
			public String name() { return "8/4"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel84();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "12"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel12();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "16"; }
			public ParticipantSelector build() {
			    return new MineField8.ParSel16();
			}
		    }
		});
	}
    }

    private class OptSelector6 extends OptSelector {
	OptSelector6() {
	    super(new SelCtorBuilder[] {
		    new SelCtorBuilder() {
			public String name() { return "3"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel3();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "6"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel6();
			}
			public boolean isDefault() { return true; }
		    },
		    new SelCtorBuilder() {
			public String name() { return "6/3"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel63();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "9"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel9();
			}
		    },
		    new SelCtorBuilder() {
			public String name() { return "12"; }
			public ParticipantSelector build() {
			    return new MineField6.ParSel12();
			}
		    },
		});
	}
    }

    private class OptBoard extends ChoiceGroup {
	private final OptSelector8 optSelector8 = new OptSelector8();
	private final OptSelector6 optSelector6 = new OptSelector6();

	OptBoard() {
	    super("Board Type", Choice.EXCLUSIVE);
	    setSelectedIndex(append("6", null), true);
	    append("8", null);
	}

	public MineField mineField(int w, int h,
				   boolean wrapX, boolean wrapY,
				   ParticipantSelector parSel) {
	    switch (getSelectedIndex()) {
	    case 0: return new MineField6(w, h, wrapX, wrapY, parSel);
	    case 1: return new MineField8(w, h, wrapX, wrapY, parSel);
	    default: return null;
	    }
	}

	public OptSelector optSelector() {
	    switch (getSelectedIndex()) {
	    case 0: return optSelector6;
	    case 1: return optSelector8;
	    default: return null;
	    }
	}
    }

    private class OptConfiguration extends ChoiceGroup {
	private final boolean[] resultArray;
	private int numOptions = 0;

	private void addOption(String name, boolean on) {
	    setSelectedIndex(append(name, null), on);
	    ++numOptions;
	}

	OptConfiguration() {
	    super ("Configuration", Choice.MULTIPLE);
	    addOption("Deterministic", true);
	    addOption("Torus", false);
	    resultArray = new boolean[numOptions];
	}

	public boolean determinismEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[0];
	}

	public boolean torusEnabled() {
	    getSelectedFlags(resultArray);
	    return resultArray[1];
	}
    }

    private final OptDifficulty optDifficulty = new OptDifficulty();
    private final OptBoard optBoard = new OptBoard();
    private final OptConfiguration optConfiguration = new OptConfiguration();

    private final MicroSweeper parent;

    public MenuForm(MicroSweeper parent) {
	super("MicroSweeper");
	this.parent = parent;

	append(optDifficulty);
	append(optBoard);
	append(optConfiguration);

	addCommand(playCommand);
	addCommand(exitCommand);

	setCommandListener(this);
	setItemStateListener(this);

	itemStateChanged(optBoard); // adds selector option
    }

    public int howManyMines() {
	return optDifficulty.howManyMines(optBoard.optSelector().parSel());
    }

    public boolean wrapX() {
	return optConfiguration.torusEnabled();
    }

    public boolean wrapY() {
	return wrapX();
    }

    public MineField mineField(int w, int h, boolean wrapX, boolean wrapY,
			       ParticipantSelector parSel) {
	return optBoard.mineField(w, h, wrapX, wrapY, parSel);
    }

    public ParticipantSelector parSel() {
	return optBoard.optSelector().parSel();
    }

    public MineFieldAcceptor acceptor() {
	return optConfiguration.determinismEnabled()
	    ? new DeterministicMineFieldAcceptor() : null;
    }

    public void commandAction(Command c, Displayable s) {
	if (c == exitCommand)
	    parent.exitAction();
	else if (c == continueCommand)
	    parent.continueGame();
	else
	    parent.playAction();
    }

    public void itemStateChanged(Item item) {
	if (item == optBoard) {
	    for (int i = 0; i < size(); ++i) {
		ChoiceGroup c = (ChoiceGroup)get(i);
		if (c instanceof OptSelector) {
		    delete(i);
		    break;
		}
	    }
	    for (int i = 0; i < size(); ++i) {
		if (get(i) == optBoard) {
		    insert(i + 1, optBoard.optSelector());
		    break;
		}
	    }
	}
    }

    private boolean gameActive = false;
    public void haveActiveGame(boolean status) {
	if (gameActive != status) {
	    if (status)
		addCommand(continueCommand);
	    else
		removeCommand(continueCommand);
	    gameActive = status;
	}
    }
}
